/* GRegex -- regular expression API wrapper around PCRE.
 * Copyright (C) 1999 Scott Wimer
 *
 * This is basically an ease of user wrapper around the functionality of
 * PCRE.
 *
 * With this library, we are, hopefully, drastically reducing the code
 * complexity necessary by making use of a more complex and detailed
 * data structure to store the regex info.  I am hoping to have a regex
 * interface that is almost as easy to use as Perl's.  <fingers crossed>
 *
 * Author: Scott Wimer <scottw@cgibuilder.com>
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * This library is free software, you can distribute it or modify it
 * under the following terms:
 *  1) The GNU General Public License (GPL)
 *  2) The GNU Library General Public License (LGPL)
 *  3) The Perl Artistic license (Artistic)
 *  4) The BSD license (BSD)
 *
 * In short, you can use this library in any code you desire, so long as
 * the Copyright notice above remains intact.  If you do make changes to
 * it, I would appreciate that you let me know so I can improve this 
 * library for everybody, but I'm not gonna force you to.
 * 
 * Please note that this library is just a wrapper around Philip Hazel's
 * PCRE library.  Please see the file 'LICENSE' in your PCRE distribution.
 * And, if you live in England, please send him a pint of good beer, his
 * library is great.
 *
 */

#include <glib.h>
#include <pcre.h>
#include <stdio.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif


/* Compile time options */
#define GREGEX_CASELESS    PCRE_CASELESS
#define GREGEX_EXTENDED    PCRE_EXTENDED
#define GREGEX_DOTALL      PCRE_DOTALL
#define GREGEX_DOLLAR_ENDONLY  PCRE_DOLLAR_END_ONLY /*ignored with multiline */
#define GREGEX_MULTILINE   PCRE_MULTILINE
#define GREGEX_UNGREEDY    PCRE_UNGREEDY /* invert greediness */

#define GREGEX_ANCHORED    PCRE_ANCHORED /* compile or match time */

/* Match time options */
#define GREGEX_NOTBOL      PCRE_NOTBOL
#define GREGEX_NOTEOL      PCRE_NOTEOL
#define GREGEX_NOTEMPTY    PCRE_NOTEMPTY
#define GREGEX_OPTIMIZE    0x0800
/* #define GREGEX_MULTIMATCH  0x1000  */

/* return codes */
#define GREGEX_NOMATCH     PCRE_ERROR_NOMATCH
#define GREGEX_NULL        PCRE_ERROR_NULL
#define GREGEX_BADOPTION   PCRE_ERROR_BADOPTION
#define GREGEX_BADMAGIC    PCRE_ERROR_BADMAGIC
#define GREGEX_ERROR_NODE  PCRE_ERROR_UNKNOWN_NODE
#define GREGEX_NOMEMORY    PCRE_ERROR_NOMEMORY

typedef struct _GRegex  GRegex;

struct _GRegex{
	gchar  *pattern;      /* the regular expression */
	gint    match;        /* boolean flag, 1 if a match was made, 0 if not */
	gint    matches;      /* number of matching sub patterns */
	gint    error;        /* boolean flag, 1 if there was an error, 0 if not */
	const char *errptr;   /* error message from PCRE on compile */
	gint    erroffset;    /* location in pattern of the error */
	gint    pos;          /* position in the string where last match left off */
};

typedef gchar * (*GRegexEvalCallback)(gchar *, gpointer);

/* Really quick outline of features... functions are preceded by 'g_regex_'
 *   new         - compile a pattern and put it in a g_regex structure
 *   free        - free up the memory used by the g_regex structure
 *   optimize    - study the pattern to make matching more efficient
 *   match       - try matching a pattern in the string
 *   match_next  - try matching pattern again in the string
 *   fetch       - fetch a particular matching sub pattern
 *   fetch_all   - get all of the matching sub patterns
 *   split       - split the string on a regex
 *   split_next  - for using split as an iterator of sorts
 *   replace     - replace occurances of a pattern with some text
 *   reuse       - clear out the structure to match against a new string
 */


/* create a new compiled regex from a pattern */
GRegex * g_regex_new(gchar * pattern, gint c_options, gint m_options);

/* clear the GRegex structure of stuff from the last match
 * this should probably be a macro rather than a function. */
void g_regex_reuse(GRegex * regex);

/* blow away a GRegex */
void g_regex_free(GRegex * regex);


/* match a regex, return true on match (the number of matches) */
gint g_regex_match(GRegex * regex, gchar * string, gint m_options);

/* match the next occurance of a regex in a string (Perl's m//g) */
gint g_regex_match_next(GRegex * regex, gchar * string, gint m_options);


/* match a pattern (not a compiled regex), return true on match (the
 * number of matches).  If the "o" option is used, the pattern string 
 * does not get recompiled.  */
gint g_regex_match_pattern(gchar * pattern, gchar * string, gint c_options,
                           gint m_options);
gint g_regex_match_pattern_gstring(GString * pattern, GString * string, 
                                    gint c_options, gint m_options);


/* get a sub expression match */
gchar * g_regex_fetch(GRegex * regex, gchar * string, int match_num);


/* get all sub expression matches.  The first gchar* in the array or list is
 * the full match string, the remainder of the gchar*'s are any substrings'
 * from capturing parentheses in the regex.  */
GList  * g_regex_fetch_all(GRegex * regex, gchar * string);
gchar ** g_regex_fetch_all_array(GRegex * regex, gchar * string);
GSList * g_regex_fetch_all_gslist(GRegex * regex, gchar * string);


/* split the string into a list of pieces, if the pattern used
 * in the regex has capturing parens, then we'll also add the elements
 * for each matching substring in the delimiter. */
GList  * g_regex_split(gchar * pattern, gchar * string, 
					   gint c_options, gint m_options, gint max_pieces);
GSList * g_regex_split_gslist(gchar * pattern, gchar * string, 
					   gint c_options, gint m_options, gint max_pieces);
gchar ** g_regex_split_array(gchar * pattern, gchar * string, gint c_options,
					   gint m_options, gint max_pieces);


/* split the string into a list of pieces, getting the next piece.
 * Useful inside of a while loop. 
 * Because we don't want to keep having to recompile the regex, we get
 * passed a GRegex pointer, not a gchar pointer. */
gchar * g_regex_split_next(GRegex * regex, gchar * string, gint c_options,
                           gint m_options);


/* replace matching substrings in string with different strings.  Like
 * Perl's s/// operator. */
gchar * g_regex_replace(GRegex * regex, gchar * string, gchar * replacement,
						gint m_options);
gchar * g_regex_replace_eval(GRegex * regex, gchar * string, 
							 GRegexEvalCallback eval, gpointer user_data,
							 gint m_options);


/* study the regular expression, in preparation for making lots of
 * comparisons with it. */
void g_regex_optimize(GRegex * regex);


/* return the integer equivalent of the option string, this should be
 * inlined. */
gint g_regex_options(GRegex * regex, gint options);


#ifdef __cplusplus
}  /* extern "C" */
#endif

